﻿Imports System.Runtime.CompilerServices
Imports System.Collections

''' <summary>
''' generische Extension-Functions erweitern eine Vielzahl von; oder sogar alle Klassen - nämlich
''' wenn ohne Typ-Einschränkung implementiert
''' </summary>
''' <remarks>
''' Wo möglich, setze ich 2 Prinzipien um: 
''' 1) Ursprünglich Public Shared Functions binde ich als Object-Functions 
''' an die Klassen, auf die sie anwendbar sind 
''' <example>Shared Integer.Parse(String) -> (String)Subject.ParseToInt()</example> 
''' 2) möglichst ein Object returnen, auf dem weitere Operationen direkt 
''' ausgeführt werden können 
''' <example>I.ConcatWith(" Text").AddTo(_Data)</example>
''' Das kann auch das "Subjekt" selbst sein ( wie bei AddTo() ). 
''' </remarks>
Public Module GenericExtensions

    <Extension()> _
    Public Function ConvertAllX(Of TInput, TOutput)( _
            ByVal Subj As TInput(), _
            ByVal Conv As Converter(Of TInput, TOutput)) As TOutput()
        Return Array.ConvertAll(Subj, Conv)
    End Function

    <Extension()> _
    Public Function CloneX(Of T As ICloneable)(ByVal Subj As T) As T
        'hat mich schon immer angek..., daß Klone einen anderen Typ hatten als ihr Template
        Return DirectCast(Subj.Clone, T)
    End Function

    <Extension()> _
    Public Function AddTo(Of T)(ByVal Subj As T, ByVal Collection As IList) As T
        'statt: Collection.Add(Obj)
        'implementiere ich: Subj.AddTo(Collection)
        'ermöglicht verkettete Aufrufe: Subj.AddTo(Col1).AddTo(Col2)
        Collection.Add(Subj)
        Return Subj
    End Function

    <Extension()> _
    Public Function BinarySearch(Of T)( _
            ByVal Subj As List(Of T), _
            ByVal Pattern As T, _
            ByVal Comparison As Comparison(Of T)) As Integer
        'ermöglicht, List(Of T).BinarySearch mit Comparisons aufzurufen, statt umständlich eine IComparer-Klasse implementieren zu müssen
        Return Subj.BinarySearch(Pattern, Comparison.ToComparer)
    End Function

    <Extension()> _
    Public Function BinarySearch(Of T)( _
            ByVal Subj As T(), _
            ByVal Pattern As T, _
            ByVal Comparison As Comparison(Of T)) As Integer
        '"Verobjektivierung" von Shared Array.BinarySearch(), außerdem Comparisons akzeptierend
        Return Array.BinarySearch(Subj, Pattern, Comparison.ToComparer)
    End Function

    ''' <summary>
    ''' konvertiert eine Comparison in ein IComparer-implementierendes Objekt
    ''' </summary>
    ''' <remarks>selbst einen Delegaten kann man also erweitern</remarks>
    <Extension()> _
    Public Function ToComparer(Of T)(ByVal Subj As Comparison(Of T)) As IComparer(Of T)
        'dieses enthebt einen der Lästigkeit, jedesmal eine IComparer-Klasse coden zu müssen, wenn das Framework diese m.E. veraltete Schnittstelle verlangt
        Return New ComparisonComparer(Of T)(Subj)
    End Function

    ''' <summary>
    ''' IComparer-implementierender Wrapper um eine Comparison 
    ''' </summary>
    Public Class ComparisonComparer(Of T) : Implements IComparer(Of T)
        Private _Comparison As Comparison(Of T)
        Public Sub New(ByVal Comparison As Comparison(Of T))
            _Comparison = Comparison
        End Sub
        Public Function Compare(ByVal x As T, ByVal y As T) As Integer _
              Implements IComparer(Of T).Compare
            Return _Comparison(x, y)
        End Function
    End Class

End Module
