'IDE-Voreinstellungen:
'Option Strict On
'Option Explicit On
'Option Infer On

Public Class frmSortAndSearchVB9

    Private _Data As New List(Of String)

    Private Sub frmSortAndSearchVB9_Load( _
            ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load
        'Zum Aufzeigen der Eigenheiten lexikalischer / numerischer Sortierung ist der Ziffernbereich von 4-8 unwesentlich
        'VB9: die Type-Inference erkennt Numbs Typ anhand der Initialisierung
        Dim Numbs = New Integer() {1, 2, 3, 9, 10, 11}
        For Each I In Numbs
            'VB9: Aufruf meiner Extensions ConcatWith() und AddTo()
            I.ConcatWith(" Text").AddTo(_Data)
            For Each II In Numbs
                I.ConcatWith(".", II, " Text").AddTo(_Data)
                For Each III In Numbs
                    I.ConcatWith(".", II, ".", III, " Text").AddTo(_Data)
                Next
            Next
        Next
        Me.btUnSort.PerformClick()
    End Sub

    Private Sub SortOptions_Click( _
            ByVal sender As Object, ByVal e As EventArgs) _
            Handles btUnSort.Click, btLexicalic.Click, btNumeric.Click
        Select Case True
            Case sender Is btUnSort
                Unsort()

            Case sender Is btLexicalic
                _Data.Sort() '(unzureichende) Standard-Sortierung

            Case sender Is btNumeric
                'benutzerdefinierte Sortierung mittels Vergleicher-Funktion (Comparison-Delegat)
                _Data.Sort(AddressOf NumericStringComparison)

        End Select
        Me.ListBox1.DataSource = Nothing
        Me.ListBox1.DataSource = _Data
        Me.ListBox1.SelectedIndices.Clear()
    End Sub

    Private Sub Search_Click(ByVal sender As Object, ByVal e As EventArgs) _
            Handles btByMatch.Click, btBinarySearch.Click
        Me.ListBox1.SelectedIndices.Clear()
        Select Case True
            Case sender Is btByMatch
                'bereinstimmungen suchen 
                'VB9: statt eines Predicates ist gleich die ganze Predicate-Function bergeben (anonyme Function, Lambda-Expression). 
                'S as String wird inferiert aus dem von FindIndex() erwarteten Typ, nmlich Predicate(Of String)
                Me.ListBox1.SelectedIndex = _Data.FindIndex( _
                   Function(S) S.LeftCut(" ") = Me.txtSearch.Text)

            Case sender Is btBinarySearch
                'findet auch die Werte, die dem Suchbegriff am nchsten kommen
                FindBinary()

        End Select
    End Sub

    Private Sub Unsort()
        Static Rnd As New Random
        For I = _Data.Count - 1 To 0 Step -1
            Dim Index = Rnd.Next(0, I)          'zuflliges Element...
            _Data.Add(_Data(Index))             'am Ende anfgen...
            _Data.RemoveAt(Index)               'am Index entfernen
        Next
    End Sub

    Private Sub FindBinary()
        'VB9: Aufruf meiner List(Of T)-Extension "BinarySearch"
        Dim Index = _Data.BinarySearch(Me.txtSearch.Text, AddressOf NumericStringComparison)
        'Ein negativer Index ist das Bit-Komplement der fiktiven **Einsortierposition**, wenn eine exakte bereinstimmung nicht gefunden wurde.
        Select Case Index
            Case Is >= 0
                Me.ListBox1.SelectedIndex = Index

            Case -1 'kleiner als alle
                Me.ListBox1.SelectedIndex = 0

            Case -_Data.Count 'grer als alle
                Me.ListBox1.SelectedIndex = _Data.Count - 1

            Case Else '2 Item markieren die Einsortierposition
                Index = Index Xor -1
                Me.ListBox1.SelectedIndices.Add(Index)
                Me.ListBox1.SelectedIndices.Add(Index - 1)

        End Select
    End Sub

    ''' <summary>
    ''' konvertiert etwa "2.11.3 Blabla" nach Integer(){2, 11, 3}
    ''' </summary>
    Private Function StringToNumbs(ByVal S As String) As Integer()
        'VB9 - If(S.LeftCut(" "), S)
        'Falls S.LeftCut() als Nothing ausgewertet wird, wird der 2. Parameter - S - verwendet
        Dim Splitted() = If(S.LeftCut(" "), S).Split("."c)
        'VB9: meine ConvertAll-Extension, kombiniert mit anonymer Converter-Function
        Return Splitted.ConvertAllX(Function(str) Integer.Parse(str))
    End Function

    ''' <summary>
    ''' Diese Function passt auf den im Framework vorgefertigten 
    ''' generischen Delegaten Comparison(Of T), und kann daher bei 
    ''' Suchen und Sortierungen als Comparison angegeben werden
    ''' </summary>
    Private Function NumericStringComparison( _
            ByVal x As String, ByVal y As String) As Integer
        Dim NumbsX = StringToNumbs(x)
        Dim NumbsY = StringToNumbs(y)
        For I = 0 To Math.Min(NumbsX.Length, NumbsY.Length) - 1
            Dim C = NumbsX(I).CompareTo(NumbsY(I))
            If C <> 0 Then Return C
        Next
        'alle verglichenen Elemente waren gleich
        'letzte Vergleichsmglichkeit: Lnge der beiden Arrays
        Return NumbsX.Length.CompareTo(NumbsY.Length)
    End Function

    Private Sub ckFilter_CheckedChanged( _
            ByVal sender As System.Object, ByVal e As System.EventArgs) _
            Handles ckFilter.CheckedChanged
        'VB9: Linq-Query:
        Dim Expr = From S In _Data Where S Like Me.txtFilter.Text
        'VB9: Lambda-Expression:
        'Dim Expr = _Data.Where(Function(S) S Like Me.txtFilter.Text)
        Me.ListBox1.DataSource = If(ckFilter.Checked, Expr.ToList, _Data)
    End Sub

End Class

